package edu.travel.commodity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.commodity.constant.BaseConstant;
import edu.travel.commodity.entity.*;
import edu.travel.commodity.mapper.ShopProductSpecMapper;
import edu.travel.commodity.service.*;
import edu.travel.commodity.utils.IdUtils;
import edu.travel.remote.dto.*;
import edu.travel.remote.vo.*;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.service.SysServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

import static edu.travel.rpc.RPCBaseResponse.error;

@Service
public class ShopProductSpecServiceImpl extends SysServiceImpl<ShopProductSpecMapper, ShopProductSpec> implements ShopProductSpecService {
    @Autowired
    private ShopProductService shopProductService;
    @Autowired
    private ShopSpecValueService shopSpecValueService;
    @Autowired
    private ShopProductSkuService shopProductSkuService;
    @Autowired
    private ShopProductImageService shopProductImageService;
    @Autowired
    private ShopProductSpecService shopProductSpecService;
    @Autowired
    private ShopProductSpecMapper shopProductSpecMapper;
    @Autowired
    private ShopSpecService shopSpecService;

    @Override
    public ProductSpecVo getShopSpec(ProductSpecDto param) {
        LambdaQueryWrapper<ShopProduct> query = Wrappers.<ShopProduct>lambdaQuery().eq(ShopProduct::getId, param.getProductId())
                .eq(ShopProduct::getCountryId, param.getCountryId());
        //商品信息
        ShopProduct one = shopProductService.getOne(query);
        if (ObjectUtil.isEmpty(one)) {
            return null;
        }

        List<ShopProductSku> list1 = shopProductSkuService.list(Wrappers.<ShopProductSku>lambdaQuery()
                .eq(ShopProductSku::getStatus, BaseConstant.BASIC_STATUS_NO_NUM)
                .eq(ShopProductSku::getProductId, one.getId()));

        if (ObjectUtil.isEmpty(list1)) {
            return null;
        }

        ProductSpecVo bean = BeanUtil.toBean(one, ProductSpecVo.class);
        //商品的全部图片
        List<ShopProductImage> list = shopProductImageService.lambdaQuery()
                .eq(ShopProductImage::getProductId, one.getId()).list();
        List<ProductImageVo> productImageVos = BeanUtil.copyToList(list, ProductImageVo.class);

        List<ProductImageVo> collect = productImageVos.stream().filter(item -> item.getIsDefault().equals(BaseConstant.BASIC_IS_DEFAULT_YES)).collect(Collectors.toList());
        bean.setImages(collect);

        Map<String, List<ProductImageVo>> imageMap = collect.stream().collect(Collectors.groupingBy(ProductImageVo::getSkuId));
        //SKU
        Set<String> skus = list1.stream().map(item -> item.getSkuId().toString()).collect(Collectors.toSet());

        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String project = request.getHeader("project");

        //sku规格和规格对应的值
        List<ShopSkuSpecValueVo> skuValue = shopProductSpecMapper.getSkuValue(skus, project);
        Map<String, List<ShopSkuSpecValueVo>> map = skuValue.stream().collect(Collectors.groupingBy(ShopSkuSpecValueVo::getSkuId));

        ArrayList<ProductSkuVo> productSkuVos = new ArrayList<>();
        for (ShopProductSku info : list1) {
            ProductSkuVo infoVo = BeanUtil.toBean(info, ProductSkuVo.class);
            infoVo.setDescription(one.getDescription());
            infoVo.setProductName(one.getProductName());
            bean.setImages(imageMap.get(info.getSkuId().toString()));
            List<ShopSkuSpecValueVo> shopSkuSpecValueVos = map.get(info.getSkuId().toString());
            infoVo.setSpecValue(shopSkuSpecValueVos);
            JSONArray objects = JSONUtil.parseArray(info.getImageUrl());
            infoVo.setImageUrls(JSONUtil.toList(objects, FileDto.class));
            productSkuVos.add(infoVo);
        }
        List<SpecSortVo> specSortVoList = getSpecSortVoList(param.getProductId());
        bean.setSpecSortVoList(specSortVoList);
        bean.setSkuList(productSkuVos);
        return bean;
    }


    public List<SpecSortVo> getSpecSortVoList(String productId) {
        List<ShopProductSpec> specs = shopProductSpecService.lambdaQuery().eq(ShopProductSpec::getProductId, productId).orderByAsc(ShopProductSpec::getSortOrder).list();
        List<SpecSortVo> list = BeanUtil.copyToList(specs, SpecSortVo.class);
        if (ObjectUtil.isEmpty(list)) {
            return list;
        }
        Set<Long> collect = specs.stream().map(ShopProductSpec::getId).collect(Collectors.toSet());
        List<ShopSpecValue> list1 = shopSpecValueService.lambdaQuery().in(ShopSpecValue::getProductSpecId, collect).list();
        for (SpecSortVo spec : list) {
            List<ShopSpecValue> collect1 = list1.stream().filter(item -> item.getProductSpecId().toString().equals(spec.getId()))
                    .sorted(Comparator.comparing(ShopSpecValue::getSortOrder)).collect(Collectors.toList());
            List<SpecValueSortVo> specValueSortVos = BeanUtil.copyToList(collect1, SpecValueSortVo.class);
            spec.setList(specValueSortVos);
        }
        return list;
    }

    @Override
    @Transactional
    public boolean insertProductSpec(List<InsertProductSpecDto> specs, ShopProduct bean) {
        ArrayList<ShopProductSpec> list = new ArrayList<>();
        ArrayList<ShopSpecValue> shopSpecValues = new ArrayList<>();
        for (InsertProductSpecDto spec : specs) {
            ShopProductSpec shopProductSpec = new ShopProductSpec();
            long id = IdUtils.getSnowflakeId();
            shopProductSpec.setId(id);
            //覆盖id后期存sku 和 规格关联
            spec.setSpecId(id + "");
            shopProductSpec.setProductId(bean.getId());
            shopProductSpec.setSpecName(spec.getSpecName());
            shopProductSpec.setSpecId(Long.valueOf(spec.getSpecId()));
            list.add(shopProductSpec);
            //规格值
            for (SpecValueDto specValue : spec.getSpecValues()) {
                ShopSpecValue shopSpecValue = new ShopSpecValue();
                long valueId = IdUtils.getSnowflakeId();
                specValue.setSpecValueId(valueId + "");
                shopSpecValue.setId(valueId);
                shopSpecValue.setProductSpecId(id);
                shopSpecValue.setSpecValue(specValue.getSpecValueName());
                shopSpecValues.add(shopSpecValue);
            }
        }
        this.saveBatch(list);
        return shopSpecValueService.saveBatch(shopSpecValues);
    }
    //通过spec_id获取规格值
    @Override
    public RPCBaseResponse<List<ProductSpecVo>> getShopSpecSortVoList(String specId) {
        //查询所有规格
        LambdaQueryWrapper<ShopProductSpec> queryWrapper = new LambdaQueryWrapper<>();
        List<ShopProductSpec> specs = super.getListLink(queryWrapper);
        //转换vo
        List<ProductSpecVo> specs1 = BeanUtil.copyToList(specs, ProductSpecVo.class);
        return new RPCBaseResponse<>(200, "SUCCESS", specs1);
    }

    //通过product_id添加规格，自动填充商品规格
    @Override
    public RPCBaseResponse<ProductSpecInsertShopSpecVo> insertOrUpdateProductSpecAndShopSpec(ProductSpecInsertShopSpecDto entity) {
        // 1. 获取与 productId 相关的所有中间表信息
        LambdaQueryWrapper<ShopProductSpec> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ShopProductSpec::getProductId, entity.getProductId());

        // 查询与 productId 相关的商品规格
        List<ShopProductSpec> shopProductSpecs = shopProductSpecMapper.selectList(queryWrapper);

        // 2. 检查是否找到了相关的商品规格
        if (shopProductSpecs == null || shopProductSpecs.isEmpty()) {
            return error("No specifications found for the provided productId");
        }

        // 3. 将 ProductSpecInsertShopSpecDto 转换为 SpecDto
        SpecDto specDto = new SpecDto();
        specDto.setSpecId(entity.getSpecId());
        specDto.setSpecName(entity.getSpecName());
        specDto.setCountryId(entity.getCountryId());

        // 4. 调用 shopSpecService 的 insertOrUpdate 方法新增或更新规格
        RPCBaseResponse<ProductSpecInsertShopSpecVo> response = shopSpecService.insertOrUpdate(specDto);

        System.out.println("response = " + response);
        // 5. 检查是否成功新增或更新规格
        if (response == null || response.getData() == null) {
            return error("Failed to insert or update shop specification");
        }

        // 6. 设置返回的规格 ID 和名称
        ProductSpecInsertShopSpecVo specVo = response.getData();
        entity.setSpecId(specVo.getId()); // 从返回结果中获取规格 ID
        entity.setSpecName(specVo.getSpecName()); // 从返回结果中获取规格名称

        // 7. 完成新增操作，保存或更新与商品相关的规格
        ShopProductSpec newShopProductSpec = new ShopProductSpec();
        newShopProductSpec.setProductId(Long.valueOf(entity.getProductId()));
        newShopProductSpec.setSpecId(Long.valueOf(entity.getSpecId()));
        newShopProductSpec.setSpecName(entity.getSpecName());

        // 保存 ShopProductSpec
        int result = shopProductSpecMapper.insert(newShopProductSpec); // 或者 update，根据业务逻辑

        // 检查插入或更新结果
        if (result <= 0) {
            return error("Failed to associate product with specification");
        }

        // 8. 返回最终的响应，返回之前的 response，或根据需要修改
        return new RPCBaseResponse<>(200, "SUCCESS", specVo);
    }

}
